/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.entity.FsSaleRevenueApprovalPrice;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.mapper.FsSaleRevenueApprovalPriceMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.param.FsSaleRevenueApprovalPriceAddParam;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.param.FsSaleRevenueApprovalPriceEditParam;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.param.FsSaleRevenueApprovalPriceIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.param.FsSaleRevenueApprovalPricePageParam;
import com.unis.financialSupervision.modular.fsSaleRevenueApprovalPrice.service.FsSaleRevenueApprovalPriceService;

import java.util.List;

/**
 * 销售收入上缴-原核定入库价格Service接口实现类
 *
 * @author QSHan
 * @date  2024/07/29 15:26
 **/
@Service
public class FsSaleRevenueApprovalPriceServiceImpl extends ServiceImpl<FsSaleRevenueApprovalPriceMapper, FsSaleRevenueApprovalPrice> implements FsSaleRevenueApprovalPriceService {

    @Override
    public Page<FsSaleRevenueApprovalPrice> page(FsSaleRevenueApprovalPricePageParam fsSaleRevenueApprovalPricePageParam) {
        QueryWrapper<FsSaleRevenueApprovalPrice> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(fsSaleRevenueApprovalPricePageParam.getSortField(), fsSaleRevenueApprovalPricePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(fsSaleRevenueApprovalPricePageParam.getSortOrder());
            queryWrapper.orderBy(true, fsSaleRevenueApprovalPricePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(fsSaleRevenueApprovalPricePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(FsSaleRevenueApprovalPrice::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FsSaleRevenueApprovalPriceAddParam fsSaleRevenueApprovalPriceAddParam) {
        FsSaleRevenueApprovalPrice fsSaleRevenueApprovalPrice = BeanUtil.toBean(fsSaleRevenueApprovalPriceAddParam, FsSaleRevenueApprovalPrice.class);
        this.save(fsSaleRevenueApprovalPrice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FsSaleRevenueApprovalPriceEditParam fsSaleRevenueApprovalPriceEditParam) {
        FsSaleRevenueApprovalPrice fsSaleRevenueApprovalPrice = this.queryEntity(fsSaleRevenueApprovalPriceEditParam.getId());
        BeanUtil.copyProperties(fsSaleRevenueApprovalPriceEditParam, fsSaleRevenueApprovalPrice);
        this.updateById(fsSaleRevenueApprovalPrice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FsSaleRevenueApprovalPriceIdParam> fsSaleRevenueApprovalPriceIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(fsSaleRevenueApprovalPriceIdParamList, FsSaleRevenueApprovalPriceIdParam::getId));
    }

    @Override
    public FsSaleRevenueApprovalPrice detail(FsSaleRevenueApprovalPriceIdParam fsSaleRevenueApprovalPriceIdParam) {
        return this.queryEntity(fsSaleRevenueApprovalPriceIdParam.getId());
    }

    @Override
    public FsSaleRevenueApprovalPrice queryEntity(Integer id) {
        FsSaleRevenueApprovalPrice fsSaleRevenueApprovalPrice = this.getById(id);
        if(ObjectUtil.isEmpty(fsSaleRevenueApprovalPrice)) {
            throw new CommonException("销售收入上缴-原核定入库价格不存在，id值为：{}", id);
        }
        return fsSaleRevenueApprovalPrice;
    }
}
