/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.entity.FsSaleRevenueDiffPrice;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.mapper.FsSaleRevenueDiffPriceMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.param.FsSaleRevenueDiffPriceAddParam;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.param.FsSaleRevenueDiffPriceEditParam;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.param.FsSaleRevenueDiffPriceIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.param.FsSaleRevenueDiffPricePageParam;
import com.unis.financialSupervision.modular.fsSaleRevenueDiffPrice.service.FsSaleRevenueDiffPriceService;

import java.util.List;

/**
 * 销售收入上缴-差额上缴Service接口实现类
 *
 * @author QSHan
 * @date  2024/07/29 15:31
 **/
@Service
public class FsSaleRevenueDiffPriceServiceImpl extends ServiceImpl<FsSaleRevenueDiffPriceMapper, FsSaleRevenueDiffPrice> implements FsSaleRevenueDiffPriceService {

    @Override
    public Page<FsSaleRevenueDiffPrice> page(FsSaleRevenueDiffPricePageParam fsSaleRevenueDiffPricePageParam) {
        QueryWrapper<FsSaleRevenueDiffPrice> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(fsSaleRevenueDiffPricePageParam.getSortField(), fsSaleRevenueDiffPricePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(fsSaleRevenueDiffPricePageParam.getSortOrder());
            queryWrapper.orderBy(true, fsSaleRevenueDiffPricePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(fsSaleRevenueDiffPricePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(FsSaleRevenueDiffPrice::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FsSaleRevenueDiffPriceAddParam fsSaleRevenueDiffPriceAddParam) {
        FsSaleRevenueDiffPrice fsSaleRevenueDiffPrice = BeanUtil.toBean(fsSaleRevenueDiffPriceAddParam, FsSaleRevenueDiffPrice.class);
        this.save(fsSaleRevenueDiffPrice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FsSaleRevenueDiffPriceEditParam fsSaleRevenueDiffPriceEditParam) {
        FsSaleRevenueDiffPrice fsSaleRevenueDiffPrice = this.queryEntity(fsSaleRevenueDiffPriceEditParam.getId());
        BeanUtil.copyProperties(fsSaleRevenueDiffPriceEditParam, fsSaleRevenueDiffPrice);
        this.updateById(fsSaleRevenueDiffPrice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FsSaleRevenueDiffPriceIdParam> fsSaleRevenueDiffPriceIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(fsSaleRevenueDiffPriceIdParamList, FsSaleRevenueDiffPriceIdParam::getId));
    }

    @Override
    public FsSaleRevenueDiffPrice detail(FsSaleRevenueDiffPriceIdParam fsSaleRevenueDiffPriceIdParam) {
        return this.queryEntity(fsSaleRevenueDiffPriceIdParam.getId());
    }

    @Override
    public FsSaleRevenueDiffPrice queryEntity(Integer id) {
        FsSaleRevenueDiffPrice fsSaleRevenueDiffPrice = this.getById(id);
        if(ObjectUtil.isEmpty(fsSaleRevenueDiffPrice)) {
            throw new CommonException("销售收入上缴-差额上缴不存在，id值为：{}", id);
        }
        return fsSaleRevenueDiffPrice;
    }
}
