/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.entity.FsSaleRevenueInspectionLoss;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.mapper.FsSaleRevenueInspectionLossMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.param.FsSaleRevenueInspectionLossAddParam;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.param.FsSaleRevenueInspectionLossEditParam;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.param.FsSaleRevenueInspectionLossIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.param.FsSaleRevenueInspectionLossPageParam;
import com.unis.financialSupervision.modular.fsSaleRevenueInspectionLoss.service.FsSaleRevenueInspectionLossService;

import java.util.List;

/**
 * 销售收入上缴-质检损益信息Service接口实现类
 *
 * @author QSHan
 * @date  2024/07/29 15:15
 **/
@Service
public class FsSaleRevenueInspectionLossServiceImpl extends ServiceImpl<FsSaleRevenueInspectionLossMapper, FsSaleRevenueInspectionLoss> implements FsSaleRevenueInspectionLossService {

    @Override
    public Page<FsSaleRevenueInspectionLoss> page(FsSaleRevenueInspectionLossPageParam fsSaleRevenueInspectionLossPageParam) {
        QueryWrapper<FsSaleRevenueInspectionLoss> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(fsSaleRevenueInspectionLossPageParam.getSortField(), fsSaleRevenueInspectionLossPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(fsSaleRevenueInspectionLossPageParam.getSortOrder());
            queryWrapper.orderBy(true, fsSaleRevenueInspectionLossPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(fsSaleRevenueInspectionLossPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(FsSaleRevenueInspectionLoss::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FsSaleRevenueInspectionLossAddParam fsSaleRevenueInspectionLossAddParam) {
        FsSaleRevenueInspectionLoss fsSaleRevenueInspectionLoss = BeanUtil.toBean(fsSaleRevenueInspectionLossAddParam, FsSaleRevenueInspectionLoss.class);
        this.save(fsSaleRevenueInspectionLoss);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FsSaleRevenueInspectionLossEditParam fsSaleRevenueInspectionLossEditParam) {
        FsSaleRevenueInspectionLoss fsSaleRevenueInspectionLoss = this.queryEntity(fsSaleRevenueInspectionLossEditParam.getId());
        BeanUtil.copyProperties(fsSaleRevenueInspectionLossEditParam, fsSaleRevenueInspectionLoss);
        this.updateById(fsSaleRevenueInspectionLoss);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FsSaleRevenueInspectionLossIdParam> fsSaleRevenueInspectionLossIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(fsSaleRevenueInspectionLossIdParamList, FsSaleRevenueInspectionLossIdParam::getId));
    }

    @Override
    public FsSaleRevenueInspectionLoss detail(FsSaleRevenueInspectionLossIdParam fsSaleRevenueInspectionLossIdParam) {
        return this.queryEntity(fsSaleRevenueInspectionLossIdParam.getId());
    }

    @Override
    public FsSaleRevenueInspectionLoss queryEntity(Integer id) {
        FsSaleRevenueInspectionLoss fsSaleRevenueInspectionLoss = this.getById(id);
        if(ObjectUtil.isEmpty(fsSaleRevenueInspectionLoss)) {
            throw new CommonException("销售收入上缴-质检损益信息不存在，id值为：{}", id);
        }
        return fsSaleRevenueInspectionLoss;
    }
}
