/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsSaleRevenueSettle.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.entity.FsSaleRevenueSettle;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.mapper.FsSaleRevenueSettleMapper;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.param.FsSaleRevenueSettleAddParam;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.param.FsSaleRevenueSettleEditParam;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.param.FsSaleRevenueSettleIdParam;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.param.FsSaleRevenueSettlePageParam;
import com.unis.financialSupervision.modular.fsSaleRevenueSettle.service.FsSaleRevenueSettleService;

import java.util.List;

/**
 * 销售收入上缴-结算信息Service接口实现类
 *
 * @author QSHan
 * @date  2024/07/29 15:29
 **/
@Service
public class FsSaleRevenueSettleServiceImpl extends ServiceImpl<FsSaleRevenueSettleMapper, FsSaleRevenueSettle> implements FsSaleRevenueSettleService {

    @Override
    public Page<FsSaleRevenueSettle> page(FsSaleRevenueSettlePageParam fsSaleRevenueSettlePageParam) {
        QueryWrapper<FsSaleRevenueSettle> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(fsSaleRevenueSettlePageParam.getSortField(), fsSaleRevenueSettlePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(fsSaleRevenueSettlePageParam.getSortOrder());
            queryWrapper.orderBy(true, fsSaleRevenueSettlePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(fsSaleRevenueSettlePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(FsSaleRevenueSettle::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FsSaleRevenueSettleAddParam fsSaleRevenueSettleAddParam) {
        FsSaleRevenueSettle fsSaleRevenueSettle = BeanUtil.toBean(fsSaleRevenueSettleAddParam, FsSaleRevenueSettle.class);
        this.save(fsSaleRevenueSettle);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FsSaleRevenueSettleEditParam fsSaleRevenueSettleEditParam) {
        FsSaleRevenueSettle fsSaleRevenueSettle = this.queryEntity(fsSaleRevenueSettleEditParam.getId());
        BeanUtil.copyProperties(fsSaleRevenueSettleEditParam, fsSaleRevenueSettle);
        this.updateById(fsSaleRevenueSettle);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FsSaleRevenueSettleIdParam> fsSaleRevenueSettleIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(fsSaleRevenueSettleIdParamList, FsSaleRevenueSettleIdParam::getId));
    }

    @Override
    public FsSaleRevenueSettle detail(FsSaleRevenueSettleIdParam fsSaleRevenueSettleIdParam) {
        return this.queryEntity(fsSaleRevenueSettleIdParam.getId());
    }

    @Override
    public FsSaleRevenueSettle queryEntity(Integer id) {
        FsSaleRevenueSettle fsSaleRevenueSettle = this.getById(id);
        if(ObjectUtil.isEmpty(fsSaleRevenueSettle)) {
            throw new CommonException("销售收入上缴-结算信息不存在，id值为：{}", id);
        }
        return fsSaleRevenueSettle;
    }
}
