/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsinterestmaintain.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.unis.common.pojo.CommonValidList;
import com.unis.financialSupervision.modular.fsinterestmaintain.entity.BusinessFsInterestMaintain;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainAddParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainEditParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainIdParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainPageParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.service.BusinessFsInterestMaintainService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import com.unis.common.annotation.CommonLog;
import com.unis.common.pojo.CommonResult;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 财务监管-利息维护控制器
 *
 * @author ZeroLiYi
 * @date  2024/07/25 15:20
 */
@Api(tags = "财务监管-利息维护控制器")
@ApiSupport(author = "UNIS_TEAM", order = 1)
@RestController
@Validated
public class BusinessFsInterestMaintainController {

    @Resource
    private BusinessFsInterestMaintainService businessFsInterestMaintainService;

    /**
     * 获取财务监管-利息维护分页
     *
     * @author ZeroLiYi
     * @date  2024/07/25 15:20
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取财务监管-利息维护分页")
    @SaCheckPermission("/finishedGrainSupervision/fsinterestmaintain/page")
    @GetMapping("/finishedGrainSupervision/fsinterestmaintain/page")
    public CommonResult<Page<BusinessFsInterestMaintain>> page(BusinessFsInterestMaintainPageParam businessFsInterestMaintainPageParam) {
        return CommonResult.data(businessFsInterestMaintainService.page(businessFsInterestMaintainPageParam));
    }

    /**
     * 添加财务监管-利息维护
     *
     * @author ZeroLiYi
     * @date  2024/07/25 15:20
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加财务监管-利息维护")
    @CommonLog("添加财务监管-利息维护")
    @SaCheckPermission("/finishedGrainSupervision/fsinterestmaintain/add")
    @PostMapping("/finishedGrainSupervision/fsinterestmaintain/add")
    public CommonResult<String> add(@RequestBody @Valid BusinessFsInterestMaintainAddParam businessFsInterestMaintainAddParam) {
        businessFsInterestMaintainService.add(businessFsInterestMaintainAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑财务监管-利息维护
     *
     * @author ZeroLiYi
     * @date  2024/07/25 15:20
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑财务监管-利息维护")
    @CommonLog("编辑财务监管-利息维护")
    @SaCheckPermission("/finishedGrainSupervision/fsinterestmaintain/edit")
    @PostMapping("/finishedGrainSupervision/fsinterestmaintain/edit")
    public CommonResult<String> edit(@RequestBody @Valid BusinessFsInterestMaintainEditParam businessFsInterestMaintainEditParam) {
        businessFsInterestMaintainService.edit(businessFsInterestMaintainEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除财务监管-利息维护
     *
     * @author ZeroLiYi
     * @date  2024/07/25 15:20
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除财务监管-利息维护")
    @CommonLog("删除财务监管-利息维护")
    @SaCheckPermission("/finishedGrainSupervision/fsinterestmaintain/delete")
    @PostMapping("/finishedGrainSupervision/fsinterestmaintain/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                               CommonValidList<BusinessFsInterestMaintainIdParam> businessFsInterestMaintainIdParamList) {
        businessFsInterestMaintainService.delete(businessFsInterestMaintainIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取财务监管-利息维护详情
     *
     * @author ZeroLiYi
     * @date  2024/07/25 15:20
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取财务监管-利息维护详情")
    @SaCheckPermission("/finishedGrainSupervision/fsinterestmaintain/detail")
    @GetMapping("/finishedGrainSupervision/fsinterestmaintain/detail")
    public CommonResult<BusinessFsInterestMaintain> detail(@Valid BusinessFsInterestMaintainIdParam businessFsInterestMaintainIdParam) {
        return CommonResult.data(businessFsInterestMaintainService.detail(businessFsInterestMaintainIdParam));
    }
}
