/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package com.unis.financialSupervision.modular.fsinterestmaintain.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unis.common.constant.DBConstant;
import com.unis.financialSupervision.modular.fsinterestmaintain.entity.BusinessFsInterestMaintain;
import com.unis.financialSupervision.modular.fsinterestmaintain.mapper.BusinessFsInterestMaintainMapper;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainAddParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainEditParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainIdParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.param.BusinessFsInterestMaintainPageParam;
import com.unis.financialSupervision.modular.fsinterestmaintain.service.BusinessFsInterestMaintainService;
import com.unis.financialSupervision.modular.fsinterestsubsidysettle.entity.FsInterestSubsidySettle;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.unis.common.enums.CommonSortOrderEnum;
import com.unis.common.exception.CommonException;
import com.unis.common.page.CommonPageRequest;

import javax.annotation.Resource;
import java.util.List;

/**
 * 财务监管-利息维护Service接口实现类
 *
 * @author ZeroLiYi
 * @date  2024/07/25 15:20
 **/
@Service
@DS(DBConstant.DS_DEPOT)
public class BusinessFsInterestMaintainServiceImpl extends ServiceImpl<BusinessFsInterestMaintainMapper, BusinessFsInterestMaintain> implements BusinessFsInterestMaintainService {
    @Resource
    private BusinessFsInterestMaintainMapper businessFsInterestMaintainMapper;

    @Override
    public Page<BusinessFsInterestMaintain> page(BusinessFsInterestMaintainPageParam businessFsInterestMaintainPageParam) {
        QueryWrapper<BusinessFsInterestMaintain> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(businessFsInterestMaintainPageParam.getSortField(), businessFsInterestMaintainPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(businessFsInterestMaintainPageParam.getSortOrder());
            queryWrapper.orderBy(true, businessFsInterestMaintainPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(businessFsInterestMaintainPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(BusinessFsInterestMaintain::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BusinessFsInterestMaintainAddParam businessFsInterestMaintainAddParam) {
        BusinessFsInterestMaintain businessFsInterestMaintain = BeanUtil.toBean(businessFsInterestMaintainAddParam, BusinessFsInterestMaintain.class);
        this.save(businessFsInterestMaintain);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BusinessFsInterestMaintainEditParam businessFsInterestMaintainEditParam) {
        BusinessFsInterestMaintain businessFsInterestMaintain = this.queryEntity(businessFsInterestMaintainEditParam.getId());
        BeanUtil.copyProperties(businessFsInterestMaintainEditParam, businessFsInterestMaintain);
        this.updateById(businessFsInterestMaintain);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BusinessFsInterestMaintainIdParam> businessFsInterestMaintainIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(businessFsInterestMaintainIdParamList, BusinessFsInterestMaintainIdParam::getId));
    }

    @Override
    public BusinessFsInterestMaintain detail(BusinessFsInterestMaintainIdParam businessFsInterestMaintainIdParam) {
        return this.queryEntity(businessFsInterestMaintainIdParam.getId());
    }

    @Override
    public BusinessFsInterestMaintain queryEntity(Integer id) {
        BusinessFsInterestMaintain businessFsInterestMaintain = this.getById(id);
        if(ObjectUtil.isEmpty(businessFsInterestMaintain)) {
            throw new CommonException("财务监管-利息维护不存在，id值为：{}", id);
        }
        return businessFsInterestMaintain;
    }

    @Override
    public List<FsInterestSubsidySettle> finfByYear(Integer planYear) {
        return businessFsInterestMaintainMapper.finfByYear(planYear);
    }
}
